@@ -16,6 +16,37 @@ fn get_function_signature(function_prototype: &str) -> [u8; 4] {
     sig
 }
 
+fn write_function_signature(sig_str: &str) -> String {
+    let re = Regex::new(r"^(?P<name>[^(]+?)\((?P<params>[^)]*?)\)").unwrap();
+    if let Some(cap) = re.captures(sig_str) {
+        let fn_name = cap.name("name").unwrap().as_str();
+        let params = cap.name("params").unwrap().as_str().replace(" ", "");
+        let canonical_fn = format!(
+            "{}({})",
+            fn_name,
+            params
+                .split(',')
+                .map(|p| {
+                    let p_split = p.split(':').collect::<Vec<_>>();
+                    if p_split.len() == 2 {
+                        p_split[1]
+                    } else {
+                        p_split[0]
+                    }
+                    .trim()
+                })
+                .collect::<Vec<_>>()
+                .join(",")
+        );
+        format!(r"{:?}", get_function_signature(&canonical_fn))
+    } else {
+        format!(
+            "{}_SIG",
+            sig_str.to_string().replace(" ", "").to_ascii_uppercase()
+        )
+    }
+}
+
 /// helps you setup the main function of a contract
 ///
 /// There are three different kind contract output.
@@ -109,7 +140,7 @@ pub fn ewasm_main(attr: TokenStream, item: TokenStream) -> TokenStream {
             #[cfg(target_arch = "wasm32")]
             use sewup::ewasm_api::finish_data;
             #[cfg(all(not(target_arch = "wasm32"), not(test)))]
-            pub fn main() { compile_error!("The function wrapped with ewasm_main need to be compiled with wasm32 target"); }
+            pub fn main() {}
             #[cfg(target_arch = "wasm32")]
             #[cfg(not(any(feature = "constructor", feature = "constructor-test")))]
             #[no_mangle]
@@ -134,7 +165,7 @@ pub fn ewasm_main(attr: TokenStream, item: TokenStream) -> TokenStream {
             #[cfg(target_arch = "wasm32")]
             use sewup::ewasm_api::finish_data;
             #[cfg(all(not(target_arch = "wasm32"), not(test)))]
-            pub fn main() { compile_error!("The function wrapped with ewasm_main need to be compiled with wasm32 target"); }
+            pub fn main() {}
             #[cfg(target_arch = "wasm32")]
             #[cfg(not(any(feature = "constructor", feature = "constructor-test")))]
             #[no_mangle]
@@ -161,7 +192,7 @@ pub fn ewasm_main(attr: TokenStream, item: TokenStream) -> TokenStream {
             #[cfg(target_arch = "wasm32")]
             use sewup::ewasm_api::finish_data;
             #[cfg(all(not(target_arch = "wasm32"), not(test)))]
-            pub fn main() { compile_error!("The function wrapped with ewasm_main need to be compiled with wasm32 target"); }
+            pub fn main() {}
             #[cfg(target_arch = "wasm32")]
             #[cfg(not(any(feature = "constructor", feature = "constructor-test")))]
             #[no_mangle]
@@ -182,7 +213,7 @@ pub fn ewasm_main(attr: TokenStream, item: TokenStream) -> TokenStream {
             #[cfg(target_arch = "wasm32")]
             use sewup::ewasm_api::finish_data;
             #[cfg(all(not(target_arch = "wasm32"), not(test)))]
-            pub fn main() { compile_error!("The function wrapped with ewasm_main need to be compiled with wasm32 target"); }
+            pub fn main() {}
             #[cfg(target_arch = "wasm32")]
             #[cfg(not(any(feature = "constructor", feature = "constructor-test")))]
             #[no_mangle]
@@ -278,7 +309,7 @@ pub fn ewasm_fn(attr: TokenStream, item: TokenStream) -> TokenStream {
         Span::call_site(),
     );
     let result = quote! {
-        pub(crate) const #sig_name : [u8; 4] = [#sig_0, #sig_1, #sig_2, #sig_3];
+        pub const #sig_name : [u8; 4] = [#sig_0, #sig_1, #sig_2, #sig_3];
         pub(crate) const #abi_info: &'static str = #abi_str;
 
         #[cfg(target_arch = "wasm32")]
@@ -473,37 +504,33 @@ pub fn ewasm_lib_fn(attr: TokenStream, item: TokenStream) -> TokenStream {
 #[proc_macro_error]
 #[proc_macro]
 pub fn ewasm_fn_sig(item: TokenStream) -> TokenStream {
-    let re = Regex::new(r"^(?P<name>[^(]+?)\((?P<params>[^)]*?)\)").unwrap();
+    write_function_signature(&item.to_string()).parse().unwrap()
+}
+
+/// helps you generate the input raw data for specific contract handler
+/// ```compile_fail
+/// let create_input = person::protocol(person.clone());
+/// let mut input = ewasm_input!(create_input for person::create);
+/// ```
+#[proc_macro]
+pub fn ewasm_input(item: TokenStream) -> TokenStream {
+    let re = Regex::new(r"(?P<instance>.*)\s+for\s+(?P<sig>.*)").unwrap();
     if let Some(cap) = re.captures(&item.to_string()) {
-        let fn_name = cap.name("name").unwrap().as_str();
-        let params = cap.name("params").unwrap().as_str().replace(" ", "");
-        let canonical_fn = format!(
-            "{}({})",
-            fn_name,
-            params
-                .split(',')
-                .map(|p| {
-                    let p_split = p.split(':').collect::<Vec<_>>();
-                    if p_split.len() == 2 {
-                        p_split[1]
-                    } else {
-                        p_split[0]
-                    }
-                    .trim()
-                })
-                .collect::<Vec<_>>()
-                .join(",")
-        );
-        format!(r"{:?}", get_function_signature(&canonical_fn))
-            .parse()
-            .unwrap()
-    } else {
+        let sig = cap.name("sig").unwrap().as_str();
+        let instance = cap.name("instance").unwrap().as_str();
         format!(
-            "{}_SIG",
-            item.to_string().replace(" ", "").to_ascii_uppercase()
+            "{{
+            let mut input = {}.to_vec();
+            input.append(&mut bincode::serialize(&{}).unwrap());
+            input
+        }}",
+            write_function_signature(sig),
+            instance
         )
         .parse()
         .unwrap()
+    } else {
+        panic!("ewasm_input input incorrect")
     }
 }
 
@@ -839,10 +866,10 @@ pub fn derive_table(item: TokenStream) -> TokenStream {
         }
         pub mod #captal_name {
             use sewup_derive::ewasm_fn_sig;
-            pub(crate) const GET_SIG: [u8; 4] = ewasm_fn_sig!(#struct_name::get());
-            pub(crate) const CREATE_SIG: [u8; 4] = ewasm_fn_sig!(#struct_name::create());
-            pub(crate) const UPDATE_SIG: [u8; 4] = ewasm_fn_sig!(#struct_name::update());
-            pub(crate) const DELETE_SIG: [u8; 4] = ewasm_fn_sig!(#struct_name::delete());
+            pub const GET_SIG: [u8; 4] = ewasm_fn_sig!(#struct_name::get());
+            pub const CREATE_SIG: [u8; 4] = ewasm_fn_sig!(#struct_name::create());
+            pub const UPDATE_SIG: [u8; 4] = ewasm_fn_sig!(#struct_name::update());
+            pub const DELETE_SIG: [u8; 4] = ewasm_fn_sig!(#struct_name::delete());
         }
 
         #[derive(Default, Clone, Copy, sewup::Serialize, sewup::Deserialize)]
@@ -1077,7 +1104,7 @@ pub fn ewasm_test(attr: TokenStream, item: TokenStream) -> TokenStream {
 
                 fn _build_runtime_and_runner() -> (
                     Arc<RefCell<TestRuntime>>,
-                    impl Fn(Arc<RefCell<TestRuntime>>, &str, [u8; 4], Option<&[u8]>, Vec<u8>) -> (),
+                    impl Fn(Arc<RefCell<TestRuntime>>, Option<&str>, &str, [u8; 4], Option<&[u8]>, Vec<u8>) -> (),
                 ) {
                     let rt = Arc::new(RefCell::new(TestRuntime::default()"#.to_string()
                             + &runtime_log_option
@@ -1096,16 +1123,17 @@ pub fn ewasm_test(attr: TokenStream, item: TokenStream) -> TokenStream {
 
                     (rt,
                         |runtime: Arc<RefCell<TestRuntime>>,
-                         fn_name: &str,
-                         sig: [u8; 4],
-                         input_data: Option<&[u8]>,
-                         expect_output: Vec<u8>| {
+                        caller: Option<&str>,
+                        fn_name: &str,
+                        sig: [u8; 4],
+                        input_data: Option<&[u8]>,
+                        expect_output: Vec<u8>| {
                             let mut h = ContractHandler {
                                 call_data: Some(_build_wasm(None)),
                                 rt: Some(runtime.clone())
                             };
 
-                            match h.execute(sig, input_data, 1_000_000_000_000) {
+                            match h.execute(caller, sig, input_data, 1_000_000_000_000) {
                                 Ok(r) => assert_eq!(r.output_data, expect_output, "{} output is unexpected", fn_name),
                                 Err(e) => {
                                     panic!("vm error: {:?}", e);
@@ -1163,26 +1191,35 @@ pub fn ewasm_test(attr: TokenStream, item: TokenStream) -> TokenStream {
 ///     }
 /// }
 /// ```
+///
+/// Besides, you can run the handler as a block chan user with `by` syntax
+/// ```compile_fail
+/// ewasm_assert_eq!(handler_fn() by "eD5897cCEa7aee785D31cdcA87Cf59D1D041aAFC", vec![74, 111, 118, 121]);
+/// ```
 #[proc_macro_error]
 #[proc_macro]
 pub fn ewasm_assert_eq(item: TokenStream) -> TokenStream {
-    let re = Regex::new(r"^(?P<fn_name>[^(]+?)\((?P<params>[^)]*?)\),(?P<equivalence>.*)").unwrap();
+    let re = Regex::new(r#"^(?P<fn_name>[^(]+?)\((?P<params>[^)]*?)\)\s*(by)?\s*(?P<caller>"[^"]*")?\s*,(?P<equivalence>.*)"#).unwrap();
     if let Some(cap) = re.captures(&item.to_string().replace("\n", "")) {
         let fn_name = cap.name("fn_name").unwrap().as_str().replace(" ", "");
         let params = cap.name("params").unwrap().as_str().replace(" ", "");
         let equivalence = cap.name("equivalence").unwrap().as_str();
+        let caller = cap
+            .name("caller")
+            .map(|c| format!("Some({})", c.as_str()))
+            .unwrap_or_else(|| "None".to_string());
         if params.is_empty() {
             format!(
-                r#"_run_wasm_fn( _runtime.clone(), "{}", ewasm_fn_sig!({}), None, {});"#,
-                fn_name, fn_name, equivalence
+                r#"_run_wasm_fn( _runtime.clone(), {}, "{}", ewasm_fn_sig!({}), None, {});"#,
+                caller, fn_name, fn_name, equivalence
             )
             .parse()
             .unwrap()
         } else {
             format!(
                 r#"_bin = bincode::serialize(&{}).unwrap();
-                   _run_wasm_fn( _runtime.clone(), "{}", ewasm_fn_sig!({}), Some(&_bin), {});"#,
-                params, fn_name, fn_name, equivalence
+                   _run_wasm_fn( _runtime.clone(), {}, "{}", ewasm_fn_sig!({}), Some(&_bin), {});"#,
+                params, caller, fn_name, fn_name, equivalence
             )
             .parse()
             .unwrap()
@@ -1196,26 +1233,31 @@ pub fn ewasm_assert_eq(item: TokenStream) -> TokenStream {
 ///
 /// This usage of the macro likes `ewasm_assert_eq`, but the contract main function should be
 /// decorated with `#[ewasm_main(auto)]`, and the equivalence arm will be serialized into `Vec<u8>`
+/// Besides, you can run the handler as a block chan user with `by` syntax as the same usage of `ewasm_assert_eq`.
 #[proc_macro_error]
 #[proc_macro]
 pub fn ewasm_auto_assert_eq(item: TokenStream) -> TokenStream {
-    let re = Regex::new(r"^(?P<fn_name>[^(]+?)\((?P<params>[^)]*?)\),(?P<equivalence>.*)").unwrap();
+    let re = Regex::new(r#"^(?P<fn_name>[^(]+?)\((?P<params>[^)]*?)\)\s*(by)?\s*(?P<caller>"[^"]*")?\s*,(?P<equivalence>.*)"#).unwrap();
     if let Some(cap) = re.captures(&item.to_string().replace("\n", "")) {
         let fn_name = cap.name("fn_name").unwrap().as_str();
         let params = cap.name("params").unwrap().as_str().replace(" ", "");
         let equivalence = cap.name("equivalence").unwrap().as_str();
+        let caller = cap
+            .name("caller")
+            .map(|c| format!("Some({})", c.as_str()))
+            .unwrap_or_else(|| "None".to_string());
         if params.is_empty() {
             format!(
-                r#"_run_wasm_fn( _runtime.clone(), "{}", ewasm_fn_sig!({}), None, sewup_derive::ewasm_output_from!({}));"#,
-                fn_name, fn_name, equivalence
+                r#"_run_wasm_fn( _runtime.clone(), {}, "{}", ewasm_fn_sig!({}), None, sewup_derive::ewasm_output_from!({}));"#,
+                caller, fn_name, fn_name, equivalence
             )
             .parse()
             .unwrap()
         } else {
             format!(
                 r#"_bin = bincode::serialize(&{}).unwrap();
-                   _run_wasm_fn( _runtime.clone(), "{}", ewasm_fn_sig!({}), Some(&_bin), sewup_derive::ewasm_output_from!({}));"#,
-                params, fn_name, fn_name, equivalence
+                   _run_wasm_fn( _runtime.clone(), {}, "{}", ewasm_fn_sig!({}), Some(&_bin), sewup_derive::ewasm_output_from!({}));"#,
+                params, caller, fn_name, fn_name, equivalence
             )
             .parse()
             .unwrap()
@@ -1238,25 +1280,37 @@ pub fn ewasm_auto_assert_eq(item: TokenStream) -> TokenStream {
 ///     }
 /// }
 /// ```
+///
+/// Besides, you can run the handler as a block chan user with `by` syntax.
+/// ```compile_fail
+/// ewasm_assert_ok!(contract_fn() by "eD5897cCEa7aee785D31cdcA87Cf59D1D041aAFC");
+/// ```
 #[proc_macro_error]
 #[proc_macro]
 pub fn ewasm_assert_ok(item: TokenStream) -> TokenStream {
-    let re = Regex::new(r"^(?P<fn_name>[^(]+?)\((?P<params>[^)]*?)\)").unwrap();
+    let re = Regex::new(
+        r#"^(?P<fn_name>[^(]+?)\((?P<params>[^)]*?)\)\s*(by)?\s*(?P<caller>"[^"]*")?\s*"#,
+    )
+    .unwrap();
     if let Some(cap) = re.captures(&item.to_string().replace("\n", "")) {
         let fn_name = cap.name("fn_name").unwrap().as_str();
         let params = cap.name("params").unwrap().as_str().replace(" ", "");
+        let caller = cap
+            .name("caller")
+            .map(|c| format!("Some({})", c.as_str()))
+            .unwrap_or_else(|| "None".to_string());
         if params.is_empty() {
             format!(
-                r#"_run_wasm_fn( _runtime.clone(), "{}", ewasm_fn_sig!({}), None, Vec::with_capacity(0));"#,
-                fn_name, fn_name
+                r#"_run_wasm_fn( _runtime.clone(), {}, "{}", ewasm_fn_sig!({}), None, Vec::with_capacity(0));"#,
+                caller, fn_name, fn_name
             )
             .parse()
             .unwrap()
         } else {
             format!(
                 r#"_bin = bincode::serialize(&{}).unwrap();
-                   _run_wasm_fn( _runtime.clone(), "{}", ewasm_fn_sig!({}), Some(&_bin), Vec::with_capacity(0));"#,
-                params, fn_name, fn_name
+                   _run_wasm_fn( _runtime.clone(), {}, "{}", ewasm_fn_sig!({}), Some(&_bin), Vec::with_capacity(0));"#,
+                params, caller, fn_name, fn_name
             )
             .parse()
             .unwrap()
@@ -1270,25 +1324,33 @@ pub fn ewasm_assert_ok(item: TokenStream) -> TokenStream {
 ///
 /// This usage of the macro likes `ewasm_assert_ok`, this only difference is that the contract main
 /// function should be decorated with `#[ewasm_main(rusty)]`.
+/// Besides, you can run the handler as a block chan user with `by` syntax as the same usage of `ewasm_assert_ok`.
 #[proc_macro_error]
 #[proc_macro]
 pub fn ewasm_rusty_assert_ok(item: TokenStream) -> TokenStream {
-    let re = Regex::new(r"^(?P<fn_name>[^(]+?)\((?P<params>[^)]*?)\)").unwrap();
+    let re = Regex::new(
+        r#"^(?P<fn_name>[^(]+?)\((?P<params>[^)]*?)\)\s*(by)?\s*(?P<caller>"[^"]*")?\s*"#,
+    )
+    .unwrap();
     if let Some(cap) = re.captures(&item.to_string().replace("\n", "")) {
         let fn_name = cap.name("fn_name").unwrap().as_str();
         let params = cap.name("params").unwrap().as_str().replace(" ", "");
+        let caller = cap
+            .name("caller")
+            .map(|c| format!("Some({})", c.as_str()))
+            .unwrap_or_else(|| "None".to_string());
         if params.is_empty() {
             format!(
-                r#"_run_wasm_fn( _runtime.clone(), "{}", ewasm_fn_sig!({}), None, vec![0, 0, 0, 0]);"#,
-                fn_name, fn_name
+                r#"_run_wasm_fn( _runtime.clone(), {}, "{}", ewasm_fn_sig!({}), None, vec![0, 0, 0, 0]);"#,
+                caller, fn_name, fn_name
             )
             .parse()
             .unwrap()
         } else {
             format!(
                 r#"_bin = bincode::serialize(&{}).unwrap();
-                   _run_wasm_fn( _runtime.clone(), "{}", ewasm_fn_sig!({}), Some(&_bin), vec![0, 0, 0, 0]);"#,
-                params, fn_name, fn_name
+                   _run_wasm_fn( _runtime.clone(), {}, "{}", ewasm_fn_sig!({}), Some(&_bin), vec![0, 0, 0, 0]);"#,
+                params, caller, fn_name, fn_name
             )
             .parse()
             .unwrap()
